#include "para.h"
static int *signal_pipe;

int para_signal_init(int *spipe)
{
	signal_pipe = spipe;
	return pipe(spipe) < 0? -1 : 1;
}

/*
 * just write one integer to signal pipe
 */
static void generic_signal_handler(int s)
{
	write(signal_pipe[1], &s, sizeof(int));
	//fprintf(stderr, "got sig %i, write returned %d\n", s, ret);
}

pid_t para_reap_child(void)
{
	int status;
	pid_t pid = waitpid(-1, &status, WNOHANG);

	if (pid <= 0)
		return pid;
	if (WIFEXITED(status))
		PARA_DEBUG_LOG("child %i exited. Exit status: %i\n", pid,
			WEXITSTATUS(status));
	else if (WIFSIGNALED(status))
		PARA_DEBUG_LOG("child %i was killed by signal %i\n", pid,
			WTERMSIG(status));
	else 
		PARA_WARNING_LOG("child %i terminated abormally\n", pid);
	return pid;
}

void para_reap_children(void)
{
	while (para_reap_child() > 0)
		; /* nothing */
}

int para_install_sighandler(int sig) 
{
	PARA_DEBUG_LOG("catching signal %d\n", sig);
	return signal(sig, &generic_signal_handler) == SIG_ERR?  -1: 1;
}

/*
 * idea taken from libdaemon
 */
int para_next_signal()
{
	int s;
	ssize_t r;

	if ((r = read(signal_pipe[0], &s, sizeof(s)) == sizeof(s)) > 0) {
		PARA_DEBUG_LOG("next signal: %d\n", s);
		return s;
	}
	return r < 0 && (errno != EAGAIN)? 0 : -1;
}
